var referenceId = Math.random().toString(35).slice(2);
function buildPaymentRequest(payments, amount) {
	const paymentRequest = payments.paymentRequest({
		countryCode: "US",
		currencyCode: "USD",
		total: {
			amount: amount,
			label: "Total",
		},
	});
	return paymentRequest;
}

async function initializeVenmo(payments, amount) {
	const paymentRequest = buildPaymentRequest(payments, amount);
	const venmoPay = await payments.venmoPay(paymentRequest, {
		redirectURL: window.location.href,
		referenceId: referenceId,
	});
	// const venmoPayButtonTarget = document.getElementById("venmo-pay");
	// await venmoPay.attach(venmoPayButtonTarget);
	await venmoPay.attach("#venmo-pay");
	document.getElementById("venmo-spinner").style.display = "none";
	return venmoPay;
}

async function loadVenmoPay() {
	var amount = document.getElementById("amount")?.value;
	var environment = document.getElementById("ppl_environment")?.value;
	let locationId = document.getElementById("ppl_location")?.value;
	console.log(
		`DOMContentLoaded: amount: ${amount}, environment: ${environment}, locationId: ${locationId}`
	);

	if (!amount || !environment || !locationId) {
		document.getElementById("payment-status-container").innerHTML =
			"<p>Missing required parameters. Finish setting up properly first</p>";
		document.getElementById("venmo-spinner").style.display = "none";
		throw new Error("Missing required parameters");
	}

	if (!window.paypal) {
		document.getElementById("payment-status-container").innerHTML =
			"<p>PayPal.js failed to load properly</p>";
		document.getElementById("venmo-spinner").style.display = "none";
		throw new Error("PayPal.js failed to load properly");
	}

	let payments;
	let appId;
	if (environment === "sandbox") {
		appId = "sandbox-sq0idb-aYar6zHAvIrhMgeKHqKxlQ";
	} else {
		appId = "sq0idp-ZXTjnM5LRS7w5XE9MaRzvQ";
	}

	try {
		console.log(appId, locationId);
		payments = PayPal.payments(appId, locationId);
		console.log("payments", payments);
	} catch (error) {
		console.error("window.PayPal.payments Error:", error);
		document.getElementById("venmo-spinner").style.display = "none";
		document.getElementById("payment-status-container").innerHTML =
			"<p>PayPal Tokenization failed<br>" + error + " window.PayPal.payments</p>";
		// throw new Error(error);
		return;
	}

	let venmoPay;
	try {
		console.log(amount, payments);
		venmoPay = await initializeVenmo(payments, amount);
	} catch (error) {
		console.error("initializeVenmo Error:", error);
		document.getElementById("payment-status-container").innerHTML =
			"<p>Initializing Venmo Pay failed<br>" + error + "</p>";
		document.getElementById("venmo-spinner").style.display = "none";
	}
	if (venmoPay) {
		console.log("initializeVenmo", venmoPay);

		try {
			venmoPay.addEventListener("ontokenization", async (event) => {
				console.debug("ontokenization", event);
				const { tokenResult, error } = event.detail;

				if (error) {
					console.error(error);
					document.getElementById("payment-status-container").innerHTML =
						"<p>PayPal Tokenization Error<br>" + error + "</p>";
					document.getElementById("venmo-spinner").style.display = "none";
					throw new Error(error);
				} else if (tokenResult.status === "OK") {
					document.getElementById("payment_token").value = tokenResult.token;
					document.getElementById("venmo-spinner").style.display = "none";
				} else {
					let errorMessage = `Tokenization failed with status: ${tokenResult.status}`;
					if (tokenResult.errors) {
						errorMessage += ` and errors: ${JSON.stringify(tokenResult.errors)}`;
					}
					console.error(errorMessage);
					document.getElementById("payment-status-container").innerHTML =
						"<p>PayPal failure after tokenization<br>" + errorMessage + "</p>";
					document.getElementById("venmo-spinner").style.display = "none";
					throw new Error(errorMessage);
				}
			});
			// venmoPay.addEventListener("ontokenization", async function ({ detail }) {
			// 	const tokenResult = detail.tokenResult;
			// 	if (tokenResult.status === "OK") {
			// 		// change value of hidden input to token
			// 		document.getElementById("payment_token").value = tokenResult.token;
			// 		document.getElementById("venmo-spinner").style.display = "none";
			// 	} else {
			// 		let errorMessage = `Tokenization failed with status: ${tokenResult.status}`;
			// 		if (tokenResult.errors) {
			// 			errorMessage += ` and errors: ${JSON.stringify(tokenResult.errors)}`;
			// 		}
			// 		console.error(errorMessage);
			// 		document.getElementById("payment-status-container").innerHTML =
			// 			"<p>PayPal Tokenization failed<br>" + errorMessage + "</p>";
			// 		document.getElementById("venmo-spinner").style.display = "none";
			// 		throw new Error(errorMessage);
			// 	}
			// });
		} catch (error) {
			console.error(error);
			document.getElementById("payment-status-container").innerHTML =
				"<p>PayPal ontokenization Error<br>" + error + "</p>";
			document.getElementById("venmo-spinner").style.display = "none";
			throw new Error(error);
		}
	}
}

window.onload = function () {
	(function () {
		var payment_method_venmo_pay = document.querySelector(
			"li.payment_method_venmo-pay"
		);
		if (payment_method_venmo_pay) {
			loadVenmoPay();
		} else {
			console.debug("payment_method_venmo-pay not found");
			setTimeout(arguments.callee, 1000); // call myself again in 1000 msecs
		}
	})();
};
