<?php if ( ! defined( 'ABSPATH' ) ) { exit; }

if ( class_exists ( 'WC_Payment_Gateway' ) ) {
	class WC_Venmo_Pay_Gateway extends WC_Payment_Gateway {

		protected $VENMO_Client_Id;
		protected $VENMO_Refresh_Token;
		protected $VENMO_Access_Token;
		protected $VENMO_Sandbox_Client_Id;
		protected $VENMO_Sandbox_Refresh_Token;
		protected $VENMO_Sandbox_Access_Token;

		public function __construct() {

			$this->id = 'venmo-pay'; // payment gateway plugin ID
			$this->icon = WCVENMO_PLUGIN_DIR_URL . 'assets/images/venmo_35.png'; // URL of the icon that will be displayed on checkout page near your gateway name
			$this->has_fields = true; // in case you need a custom form
			$this->method_title = 'Venmo Pay by PayPal';
			$this->method_description = `<p>Venmo Pay is the official integration for PayPal merchants.</p>`; // will be displayed on the options page

			$this->init_settings();
			$this->enabled = $this->get_option( 'enabled' );
			$this->title = $this->get_option( 'checkout_title' ) ? $this->get_option( 'checkout_title' ) : $this->method_title;
			$this->description = $this->get_option( 'checkout_description' ) ? $this->get_option( 'checkout_description' ) : 'Use Venmo or your phone camera to scan the QR code to pay with Venmo';
			$this->VENMO_Client_Id = $this->get_option( 'VENMO_Client_Id' );
			$this->VENMO_Refresh_Token = $this->get_option( 'VENMO_Refresh_Token' );
			$this->VENMO_Access_Token = $this->get_option( 'VENMO_Access_Token' );
			$this->VENMO_Location_Id = $this->get_option( 'VENMO_Location_Id' );
			$this->enableSandbox = $this->get_option( 'enableSandbox' ) ?? 'no';
			$this->VENMO_Sandbox_Client_Id = '';
			$this->VENMO_Sandbox_Refresh_Token = '';
			$this->VENMO_Sandbox_Access_Token = '';
			$this->VENMO_Sandbox_Location_Id = '';
			$this->toggleTutorial = $this->get_option( 'toggleTutorial' );

			global $venmo_fs;
			$upgrade_url = venmo_fs()->get_upgrade_url();
			$pro = ' <a style="text-decoration:none" href="' .$upgrade_url . '" target="_blank"><sup style="color:red">PRO</sup></a>';
			$edit_with_pro = ' <a style="text-decoration:none" href="' .$upgrade_url . '" target="_blank">APPLY CHANGES WITH PRO</a>';

			$paypal = ' <a href="https://paypal.theafricanboss.com/access.php" target="_blank">Get it here</a>';
			$this->form_fields = array(
				'enabled' => array(
					'title'       => 'Enable Venmo Pay',
					'label'       => 'Check to Enable / Uncheck to Disable',
					'type'        => 'checkbox',
					'default'     => 'no'
				),
				'checkout_title'  => array(
					'title'       => 'Checkout Title',
					'type'        => 'text',
					'description' => 'This is the title which the user sees on the checkout page.',
					'default'     => 'Venmo Pay by PayPal',
					'placeholder' => 'Venmo Pay by PayPal',
				),
				'checkout_description' => array(
					'title'       => 'Checkout Notice',
					'type'        => 'textarea',
					'description' => 'This is the description which the user sees during checkout.',
					'default'     => 'Use Venmo or your phone camera to scan the QR code to pay with Venmo',
					'placeholder'     => 'Use Venmo or your phone camera to scan the QR code to pay with Venmo',
				),
				'VENMO_Client_Id' => array(
					'title'       => 'PayPal Merchant ID<br>' . $paypal,
					'type'        => 'text',
					'description' => 'This is your PayPal Merchant ID.' . $paypal,
					'placeholder' => 'MXXXXXXXXXXXX',
				),
				'VENMO_Refresh_Token' => array(
					'title'       => 'PayPal Refresh Token<br>' . $paypal,
					'type'        => 'password',
					'description' => 'This is your PayPal Refresh Token.' . $paypal,
					'placeholder' => 'token',
				),
				'VENMO_Access_Token' => array(
					'title'       => 'PayPal Access Token<br>' . $paypal,
					'type'        => 'password',
					'description' => 'This is your PayPal Access Token.' . $paypal,
					'placeholder' => 'token',
				),
				'VENMO_Location_Id' => array(
					'title'       => 'PayPal Location ID<br>' . $paypal,
					'type'        => 'text',
					'description' => 'This is your PayPal Location ID.' . $paypal,
					'placeholder' => 'LXXXXXXXXXXXX',
				),
				// 'enableSandbox' => array(
				// 	'title'       => 'Enable Sandbox for testing purposes',
				// 	'label'       => 'Check to Enable / Uncheck to Disable',
				// 	'type'        => 'checkbox',
				// 	'description' => 'This is for testing purposes only. Payments will not be processed.',
				// 	'default'     => 'no',
				// ),
				'toggleTutorial' => array(
					'title'       => 'Enable Tutorial on checkout',
					'label'       => 'Check to Enable / Uncheck to Disable',
					'type'        => 'checkbox',
					'description' => 'Help your customers checkout with ease',
					'default'     => 'no',
				),
			);

			// Gateways can support subscriptions, refunds, saved payment methods
			$this->supports = array('products');

			// This action hook saves the settings
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );

			// We need custom JavaScript to obtain a token
			add_action( 'wp_enqueue_scripts', array( $this, 'wc_venmo_pay_payment_scripts' ) );

			// // Thank you page
			// add_action( 'woocommerce_thankyou_venmo', array( $this, 'wc_venmo_pay_thankyou_page' ) );

			// Admin Menu
			add_action('admin_menu', array( $this, 'wc_venmo_pay_admin_menu' ) );
		}

		public function wc_venmo_pay_admin_menu() {
			$capability = 'manage_options';
			$contact_url = "mailto:info@theafricanboss.com?subject=WC%20Venmo%20Plugin%20Support&body=Hello%20Jean%2C%0D%0A%0D%0A";
			$venmo_receipts = admin_url('edit.php?post_type=venmo-receipts');
			global $venmo_fs;
			$account_url = $venmo_fs->get_account_url();

			$new = ' <sup style="color:#9f9">NEW</sup>';
			$paypal_parent_slug = 'wc-settings&tab=checkout&section=venmo-pay';
			add_submenu_page( 'woocommerce', 'Venmo Pay by PayPal', 'Venmo Pay' . $new, 'manage_woocommerce', $paypal_parent_slug, null );

			add_menu_page( null, 'Venmo Pay' . $new, $capability , $paypal_parent_slug, array( $this, 'wc_venmo_pay_admin_menu' ), 'dashicons-money-alt', 56 );
			add_submenu_page( $paypal_parent_slug , 'PayPal' , 'PayPal Tokens' . $new , $capability , 'wc_venmo_paypal' , array( $this, 'wc_venmo_paypal_menu_page' ), null );
			add_submenu_page( $paypal_parent_slug , 'Review Venmo Pay' , 'Review' , $capability , 'https://wordpress.org/support/plugin/momo-venmo/reviews/?filter=5' , null, null );
			add_submenu_page( $paypal_parent_slug , 'Account' , 'Account' , $capability , $account_url , null , null );
			add_submenu_page( $paypal_parent_slug , 'Our Most Recommended Plugins' , 'Recommended Plugins' , $capability , 'wc_venmo_recommended' , array( $this, 'wc_venmo_recommended_menu_page'), null );
			add_submenu_page( $paypal_parent_slug , 'Our Plugins' , 'Free Plugins' , $capability , admin_url("plugin-install.php?s=theafricanboss&tab=search&type=author") 	, null, null );
			add_submenu_page( $paypal_parent_slug , 'Help' , 'Help' , $capability , 'wc_venmo_help' , array( $this, 'wc_venmo_help_menu_page'), null );
			add_submenu_page( $paypal_parent_slug , 'Contact' , 'Support' , $capability , $contact_url , null , null );
		}

		public function wc_venmo_paypal_menu_page() {
			require_once WCVENMO_PLUGIN_DIR . 'includes/admin/paypal.php';
		}
		public function wc_venmo_recommended_menu_page() {
			require_once WCVENMO_PLUGIN_DIR . 'includes/admin/recommended.php';
		}

		public function wc_venmo_help_menu_page() {
			require_once WCVENMO_PLUGIN_DIR . 'includes/admin/help.php';
		}


		// /**
		//  * Logging method.
		//  *
		//  * @param string $message Log message.
		//  * @param string $level Optional. Default 'info'. Possible values:
		//  *                      emergency|alert|critical|error|warning|notice|info|debug.
		//  */
		// public static function log( $message, $level = 'info' ) {
		// 	if ( self::$log_enabled ) {
		// 		if ( empty( self::$log ) ) {
		// 			self::$log = wc_get_logger();
		// 		}
		// 		self::$log->log( $level, $message, array( 'source' => 'venmo_pay' ) );
		// 	}
		// }

		// /**
		//  * Check if this gateway is available in the user's country based on currency.
		//  *
		//  * @return bool
		//  */
		// public function is_valid_for_use() {
		// 	return in_array(
		// 		get_woocommerce_currency(),
		// 		apply_filters(
		// 			'woocommerce_venmo_pay_supported_currencies',
		// 			array( 'AUD', 'BRL', 'CAD', 'MXN', 'NZD', 'HKD', 'SGD', 'USD', 'EUR', 'JPY', 'TRY', 'NOK', 'CZK', 'DKK', 'HUF', 'ILS', 'MYR', 'PHP', 'PLN', 'SEK', 'CHF', 'TWD', 'THB', 'GBP', 'RMB', 'RUB', 'INR' )
		// 		),
		// 		true
		// 	);
		// }

		// /**
		//  * Admin Panel Options.
		//  * - Options for bits like 'title' and availability on a country-by-country basis.
		//  *
		//  * @since 1.0.0
		//  */
		// public function admin_options() {
		// 	if ( $this->is_valid_for_use() ) {
		// 		parent::admin_options();
		// 	} else {
		 		// echo '<div class="inline error"><p><strong>' . esc_html_e( 'Gateway disabled', 'momo-venmo' ) . '</strong>:' .
				// 		esc_html_e( 'Venmo Pay does not support your store currency.', 'momo-venmo' ) .
				// 		'</p></div>';
			// }
		// }

		// Payment Custom JS and CSS
		public function wc_venmo_pay_payment_scripts() {
			if ( is_checkout() ) {
				// WCVENMO_PLUGIN_DIR_URL . 'assets/css/spinner.css'
				if ($this->enableSandbox == 'yes') {
					$paypal_web = "https://sandbox.web.paypalcdn.com/v1/paypal.js";
				} else {
					$paypal_web = "https://web.paypalcdn.com/v1/paypal.js";
				}
				wp_enqueue_script( 'wc_venmo_pay_paypal_web', $paypal_web );

				// $paypal_ver  = date("ymd-Gis", filemtime( plugin_dir_path( __FILE__ ) . 'assets/js/paypal.js' ));
				// $spinner_ver = date("ymd-Gis", filemtime( plugin_dir_path( __FILE__ ) . 'assets/css/spinner.css' ));

				// wp_enqueue_script( 'wc_venmo_pay_paypal', plugins_url( 'assets/js/paypal.js', array(), $paypal_ver );
				// wp_register_style( 'wc_venmo_pay_spinner', plugins_url( 'assets/css/spinner.css', false, $spinner_ver );

				wp_enqueue_script( 'wc_venmo_pay_paypal', WCVENMO_PLUGIN_DIR_URL . 'assets/js/paypal.js' );
				wp_register_style( 'wc_venmo_pay_spinner', WCVENMO_PLUGIN_DIR_URL . 'assets/css/spinner.css' );
				wp_enqueue_style ( 'wc_venmo_pay_spinner' );
			}
			// // we need JS or CSS to process a token only on cart/checkout pages
			// if ( ! is_cart() || ! is_checkout() || ! isset( $_GET['pay_for_order'] ) ) {
			//     wp_enqueue_script( '' );
			//     return;
			// }
		}

		// Checkout page
		public function payment_fields () {
			// https://developer.paypal.com/docs/checkout/pay-with-venmo/integrate/

			global $woocommerce, $total, $amount;
			$woocommerce->cart->get_cart();
			$total = $woocommerce->cart->get_total(); // $1.00
			$amount = $woocommerce->cart->total; // 1.00
			$ppl_environment = $this->enableSandbox == 'yes' ? Environment::SANDBOX : Environment::PRODUCTION;
			$ppl_location = $this->enableSandbox == 'yes' ? $this->VENMO_Sandbox_Location_Id : $this->VENMO_Location_Id;

			echo '<fieldset id="wc-' , esc_attr( $this->id ) , '-form">';
			do_action( 'woocommerce_form_start' , $this->id );

			if ( !empty($this->checkout_description) ) {
				echo '<p>' , wp_kses_post( __( $this->checkout_description, 'momo-venmo' ) ) . '</p>';
			} else {
				echo '<p>' , wp_kses_post( __( 'Use Venmo or your phone camera to scan the QR code to pay with Venmo', 'momo-venmo' ) ) . '</p>';
			}

			echo '<div class="d-flex align-items-center">

					<input type="hidden" id="payment_token" name="payment_token" value="">
					<input type="hidden" id="amount" name="amount" value="' . $amount . '">
					<input type="hidden" id="ppl_environment" name="ppl_environment" value="' . $ppl_environment . '">
					<input type="hidden" id="ppl_location" name="ppl_location" value="' . $ppl_location . '">

					<form id="payment-form">
					<div id="venmo-spinner" class="d-flex justify-content-center align-items-center">
					<span class="spinner-grow text-dark" role="status"></span>
					</div>
					<div class="d-flex justify-content-end" id="venmo-pay"></div>
					</form>
					<div class="d-flex justify-content-center" id="payment-status-container"></div>
				</div>';
			// toggleTutorial
			if ( 'yes' === $this->toggleTutorial ) {
				// echo '<p><img src="//images.ctfassets.net/1nw4q0oohfju/4UPvCW9VxdWn9nXsW7kZ5R/492a888883fd52eebcc2be1a52f51b32/venmo-pay-scan-to-pay.gif" alt="A screenshot shows the dialog box of the approved Venmo linked account."></p>';
			}

			do_action( 'woocommerce_form_end' , $this->id );
			echo '<div class="clear"></div></fieldset>';
		}

		// validate payment token
		public function validate_fields() {
			$ppl_payment_token = sanitize_text_field(trim($_POST[ 'payment_token' ]));
			if ( !$ppl_payment_token || strlen($ppl_payment_token) < 5 ) {
				wc_add_notice( esc_html( __('Bad PayPal Token. Please Refresh', 'momo-venmo' ) ), 'error' );
			}
		}

		// Thank you page
		public function wc_venmo_pay_thankyou_page( $order_id ) {
			global $woocommerce;
			$order = wc_get_order( $order_id );
			if ( 'venmo-pay' === $order->get_payment_method() ) {
				$paypal_receipt = "#";
				$note = wp_kses_post( "<p>" . sprintf( __('Here is your <a href="%s" target="blank">PayPal receipt</a>', 'momo-venmo'), $paypal_receipt ) . "</p>" );
			}
		}

		// Process Order
		public function process_payment( $order_id ) {
			global $woocommerce;
			$order = wc_get_order( $order_id );
			// $amount = $woocommerce->cart->total; // 1.00
			$amount = $order->total; // 1.00
			$currency = $order->get_currency();

			$amount_money = new Money();
			$amount_money->setAmount(floatval($amount) * 100);
			$amount_money->setCurrency($currency);
			$app_fee_money = new Money();
			$app_fee_money->setAmount(floatval($amount));
			$app_fee_money->setCurrency($currency);

			$ppl_payment_token = sanitize_text_field(trim($_POST['payment_token']));
			// $ppl_location = $this->enableSandbox == 'yes' ? $this->VENMO_Sandbox_Location_Id : $this->VENMO_Location_Id;
			// $ppl_merchant_id = $this->enableSandbox == 'yes' ? $this->VENMO_Sandbox_Client_Id : $this->VENMO_Client_Id;

			if( ! is_wp_error($order) ) {
				$paymentRequest = new CreatePaymentRequest(
					$ppl_payment_token,
					$order_id,
					$amount_money
				);

				$paymentRequest->setAppFeeMoney($app_fee_money);
				$paymentRequest->setAutocomplete(true);
				$paymentRequest->setReferenceId($order_id);
				$paymentRequest->setNote('Payment for order #' . $order_id);

				// Initialize PayPal PHP SDK OAuth API client
				$ppl_environment = $this->enableSandbox == 'yes' ? Environment::SANDBOX : Environment::PRODUCTION;
				$ppl_access_token = $this->enableSandbox == 'yes' ? $this->VENMO_Sandbox_Access_Token : $this->VENMO_Access_Token;
				$apiClient = new PayPalClient([
					'environment' => $ppl_environment,
					'accessToken' => $ppl_access_token
				]);

				try {
					$api_response = $apiClient->getPaymentsApi()->createPayment($paymentRequest);
					if ($api_response->isSuccess()) {
						$result = $api_response->getResult();
						$order->payment_complete();

						// if ( 'venmo-pay' === $order->get_payment_method() ) {
						// 	$paypal_receipt = $result->payment->receipt_url;
						// 	$note = wp_kses_post( "<p>" . sprintf( __('Here is your <a href="%s" target="blank">PayPal receipt</a>', 'momo-venmo'), $paypal_receipt ) . "</p>" );
						// 	$order->add_order_note( $note , true );
						// }

						$order->reduce_order_stock();

						// Empty cart
						$woocommerce->cart->empty_cart();

						// Redirect to the thank you page
						return array( 'result' => 'success', 'redirect' => $this->get_return_url( $order ) );

					} else {
						$errors = $api_response->getErrors();
						// print_r($errors);
						wc_add_notice( " " . $errors, 'error' );
						// wc_add_notice( "PayPal Error. Please try again", 'error' );
						return;
					}
				} catch (\Throwable $th) {
					// print_r($th);
					wc_add_notice( " " . $th, 'error' );
					return;
				}
			} else {
				wc_add_notice(  'Connection error.', 'error' );
				return;
			}

		}

		// //Webhook
		// public function webhook() {
		// 	$order = wc_get_order( $_GET['id'] );
		// 	$order->payment_complete();
		// 	update_option('webhook_debug', $_GET);
		// }

	}

} else {
	require_once WCVENMO_PLUGIN_DIR . 'includes/notifications/woocommerce.php';
}

?>