<?php if ( ! defined( 'ABSPATH' ) ) { exit; }

use Automattic\WooCommerce\Client;

if ( !class_exists( 'WC_Venmo_Update_Order' ) && class_exists( 'WC_Venmo_Gateway' ) ):
class WC_Venmo_Update_Order extends WC_Venmo_Gateway {

  function register() {
      add_action( 'init', array( $this, 'wc_venmo_cpt' ) );
      add_action( 'rest_api_init', array( $this, 'wc_venmo_update_order_route' ) );
  }

  // Create CPT
  function wc_venmo_cpt() {
    if ( class_exists( 'Woocommerce' ) && !post_type_exists( 'venmo-receipts' ) ) {
      register_post_type( 'venmo-receipts',
      array(
        'labels' => array(
            'name' => __( 'Venmo Receipts' ),
            'singular_name' => __( 'Venmo Receipt' )
        ),
        'public' => false,
        'show_ui' => true,
        'show_in_rest' => false,
        'has_archive' => false,
        'rewrite' => array('slug' => 'venmo-receipts'),
        'show_in_rest' => false,
        'menu_icon' => 'dashicons-money-alt',
        'menu_position' => 56,
        )
      );
    }
  }

  // Create REST API route
  function wc_venmo_update_order_route() {
    register_rest_route( 'wc-venmo/v1', '/update-venmo-order', array(
      'methods' => 'POST',
      'callback' => array( $this, 'wc_venmo_emrcpts_order_update'),
      'permission_callback' => '__return_true',
    ) );
    // register_rest_route( 'wc-venmo/v1', '/cloudmailin/update-venmo-order', array(
    //   'methods' => 'POST',
    //   'callback' => array( $this, 'wc_venmo_cloudmailin_order_update'),
    //   'permission_callback' => '__return_true',
    // ) );
  }

  // Update order
  function wc_venmo_emrcpts_order_update( $data ) {
    // header("Content-type: application/json");
    header("Content-type: text/plain");

    $body = $data->get_body_params();
    // print_r($body);

    $venmo_username = wp_kses_post($body['transactionaccountid']);
    $money = wp_kses_post($body['transactionamount']);
    $currency = wp_kses_post($body['transactioncurrency']);
    $amount = wp_kses_post($body['transactionamount']);
    $order_id = wp_kses_post($body['transactionorderid']);
    $note = wp_kses_post($body['transactionnote']);
    $receipt_post_id = null;

    $origin = get_bloginfo('url');
    echo "\nResponse by: $origin\n";

    echo "Venmo Username: $venmo_username\n";
    echo "Money: $money\n";
    echo "Currency: $currency\n";
    echo "Amount: $amount\n";
    echo "Order ID: $order_id\n";
    echo "Note: $note\n";

    if ( $_SERVER['HTTP_HOST'] !== "emailreceipts.io" ) {
      $post_dump = print_r($body, true);
      $email_subject = null;
      $shop = wp_kses_post(get_bloginfo('url'));
      $amount = wp_kses_post(floatval($amount)); // $amount == $orderamount
      $order = $this->wc_venmo_find_venmo_order($money, $amount, $order_id, $venmo_username, $email_subject, $receipt_post_id);
      require_once WCVENMO_PLUGIN_DIR . 'includes/admin/process-venmo.php';
    } else {
      http_response_code(422);
    }
    echo "Status: " . http_response_code();
  }

  function wc_venmo_find_venmo_order( $money, $amount, $order_id = null, $venmo_username = null, $email_subject = null, $receipt_post_id = null ) {
    $order = array();
    $post_title = null;
    $post_content = null;
    if (!empty($order_id)) {
      $order = wc_get_order( $order_id );
      $order_id = !empty($order) ? $order->get_id() : $order_id;
      $venmo_username = empty($venmo_username) && $order ? $order->get_meta('venmo_username') : $venmo_username;
      $post_title = "Receipt: $money from $venmo_username for $order_id";
      $post_content = "$money from $venmo_username for $order_id.";
    }

    if (empty($order)) {
      // 'orderby' => '<' . ( time() - 3600 ), // ordered before the last hour
      $orders = wc_get_orders( ['limit' => 3, 'payment_method' => 'venmo', 'orderby' => time() - 3600, 'status' => array('wc-on-hold')] );
      // print_r($orders);
      $ordercountmsg = count($orders) . " recent order(s) match(es) your criteria: payment_method: venmo, ordered in the last hour, status: on-hold\n";
      echo $ordercountmsg;
      if (count($orders) > 0) {
          $order = $orders[0];
          $orderamount = wp_kses_post(floatval($order->get_total()));
          echo "Recent order: $orderamount vs provided: $amount\n";
          if ($order->get_payment_method() === 'venmo' && $amount == $orderamount) {
            $post_title = "Receipt: $money from $venmo_username for $order_id (extracted from recent Venmo order)";
            $post_content = "$money from $venmo_username for $order_id.";
            $order_id = !empty($order) ? $order->get_id() : $order_id;
            $venmo_username = empty($venmo_username) ? $order->get_meta('venmo_username') : $venmo_username;
            echo "Recent Venmo order $order_id with venmo_username: $venmo_username matched amount $amount == $orderamount\n";
          } else {
              $post_title = "Receipt: Invalid {$order->get_payment_method_title()} order";
              $post_content = "Since the order information was invalid, we tried looking for the most recent order to see if it was a match.<br>Invalid recent order {$order->get_id()} did not match amount or payment method.<br>{$order->get_payment_method_title()} Order of amount $orderamount";
              echo "Invalid {$order->get_payment_method_title()} recent order {$order->get_id()} of amount $orderamount != $amount \n";
          }
      } else {
          $post_title = "Receipt: No valid orders matched the amount: $amount";
          $post_content = "Since the order information was invalid, we tried looking for the most recent order to see if it was a match.<br>" . $ordercountmsg;
      }
    }

    if ($post_title && $post_content && post_type_exists( 'venmo-receipts' ) ) {
        $venmo_receipt = array(
            'post_title' => $post_title,
            'post_content' => "$post_content.<br><br>$email_subject",
            'post_type' => 'venmo-receipts',
            'post_status' => 'private',
        );
        $receipt_post_id = wp_insert_post( $venmo_receipt );
        if ($receipt_post_id) {
            echo "Venmo Receipt ID: $receipt_post_id created successfully\n";
            http_response_code(201);
        } else {
            echo "Venmo Receipt creation failed\n";
            http_response_code(500);
        }
    }

    return $order;
  }

  // // Update order
  // function wc_venmo_cloudmailin_order_update( $data ) {
  //   // Setting content-type: text/plain will allow CloudMailin to store this response if it's an error
  //   header("Content-type: text/plain");

  //   $json = $data->get_json_params();
  //   // print_r($json);

  //   $headers = $json['headers'];
  //   $email_subject = wp_kses_post($headers['subject']);
  //   $sender_email = wp_kses_post($headers['from']);
  //   $receiver_email = wp_kses_post($headers['to']);

  //   $email_body = wp_kses_post($json['plain']);
  //   $forwarding_email = wp_kses_post($json['envelope']['to']);

  //   echo "Sender email: " . $sender_email . "\n";
  //   echo "Receiver email: " . $receiver_email . "\n";
  //   echo "Email subject: " . $email_subject . "\n";
  //   echo "Email body: " . $email_body . "\n";
  //   echo "Forwarding email: " . $forwarding_email . "\n";

  //   $venmo_username = null;
  //   $money = null;
  //   $currency = null;
  //   $amount = null;
  //   $order_id = null;
  //   $receipt_post_id = null;

  //   if ( $this->wc_venmo_check_email( $sender_email, $receiver_email ) && $this->VenmoForwardingEmail === $forwarding_email ) {
  //     if (preg_match('/\bpaid you\b/', $email_subject) || str_contains($email_subject, 'paid you')) {
  //       $subject_array = explode(' paid you ', $email_subject);
  //       $venmo_username = wp_kses_post($subject_array[0]);
  //       $venmo_username = wp_kses_post(str_replace ('Fwd: ', '', $venmo_username));
  //       $money_array = explode(' for ', $subject_array[1]);
  //       $money = wp_kses_post($money_array[0]);
  //       $amount = preg_match_all('/\d+/', $money, $matches);
  //       $amount = $matches[0][0];
  //       $amount = wp_kses_post(intval($amount));
  //       $order_id = wp_kses_post($money_array[1]);

  //       if ($venmo_username && $money && $order_id) {
  //         if ( post_type_exists( 'venmo-receipts' ) ) {
  //           $venmo_receipt = array(
  //             'post_title' => "Receipt: {$money} from {$venmo_username} for {$order_id}",
  //             'post_content' => "{$money} from {$venmo_username} for {$order_id}",
  //             'post_type' => 'venmo-receipts',
  //             'post_status' => 'private',
  //           );
  //           $receipt_post_id = wp_insert_post( $venmo_receipt );
  //           if ($receipt_post_id) {
  //             http_response_code(201);
  //           } else {
  //             http_response_code(500);
  //           }
  //         }
  //       } else if ($venmo_username && $money) {
  //         if ( post_type_exists( 'venmo-receipts' ) ) {
  //           $venmo_receipt = array(
  //             'post_title' => "Receipt: {$money} from {$venmo_username} for unknown order",
  //             'post_content' => "{$money} from {$venmo_username} for unknown order. Received Email Subject: {$email_subject}",
  //             'post_type' => 'venmo-receipts',
  //             'post_status' => 'private',
  //           );
  //           $receipt_post_id = wp_insert_post( $venmo_receipt );
  //           if ($receipt_post_id) {
  //             http_response_code(201);
  //           } else {
  //             http_response_code(500);
  //           }
  //         }
  //       } else {
  //         http_response_code(404);
  //       }

  //       if ( venmo_fs()->is_plan__premium_only('pro') ) {
  //         if ( $venmo_username && $amount && $order_id ) {
  //           echo wp_kses_post(" <br> {$money} from {$venmo_username} for {$order_id}");
  //           $shop = wp_kses_post(get_bloginfo('url'));
  //           $order = wc_get_order( $order_id );
  //           if ( $order ) {
  //             $order_amount = wp_kses_post(intval($order->get_total()));
  //             if ( $amount == $order_amount && $order->get_status() != 'processing' ) {
  //               $update_status = $order->update_status( 'processing' );
  //               if ( $update_status ) {
  //                 echo wp_kses_post(" <br> Order {$order_id} updated to processing");
  //                 if ( venmo_fs()->is_plan__premium_only('pro') && $this->VenmoStockManagement == 'on' && $this->VenmoForwardingEmail ) {
  //                   $order->reduce_order_stock();
  //                 }
  //                 if ( $receipt_post_id ) {
  //                   $receipt_post = get_post( $receipt_post_id );
  //                   if ( $receipt_post ) {
  //                     $receipt_post->post_title = wp_kses_post($receipt_post->post_title . " - processing");
  //                     $receipt_post->post_content = wp_kses_post($receipt_post->post_content . " - Order {$order_id} updated to processing");
  //                     wp_update_post( $receipt_post );
  //                     http_response_code(200);
  //                   }
  //                 }
  //                 http_response_code(200);
  //               } else {
  //                 echo wp_kses_post(" <br> Order {$order_id} could not be updated to processing");
  //                 if ( $receipt_post_id ) {
  //                   $receipt_post = get_post( $receipt_post_id );
  //                   if ( $receipt_post ) {
  //                     $receipt_post->post_title = wp_kses_post($receipt_post->post_title . " - status update error");
  //                     $receipt_post->post_content = wp_kses_post($receipt_post->post_content . " - Order {$order_id} could not be updated to processing");
  //                     wp_update_post( $receipt_post );
  //                     http_response_code(200);
  //                   }
  //                 }
  //                 http_response_code(500);
  //               }
  //             } else if ( $amount != $order_amount ) {
  //               echo wp_kses_post(" <br> Order {$order_id} amount does not match with {$money} provided");
  //               if ( $receipt_post_id ) {
  //                 $receipt_post = get_post( $receipt_post_id );
  //                 if ( $receipt_post ) {
  //                   $receipt_post->post_title = wp_kses_post($receipt_post->post_title . " - amount error");
  //                   $receipt_post->post_content = wp_kses_post($receipt_post->post_content . " - Order {$order_id} amount $order_amount does not match with $amount provided");
  //                   wp_update_post( $receipt_post );
  //                   http_response_code(200);
  //                 }
  //               }
  //               http_response_code(500);
  //             } else if ( $order->get_status() == 'processing' ) {
  //               echo wp_kses_post(" <br> Order {$order_id} is already processing");
  //               if ( $receipt_post_id ) {
  //                 $receipt_post = get_post( $receipt_post_id );
  //                 if ( $receipt_post ) {
  //                   $receipt_post->post_title = wp_kses_post($receipt_post->post_title . " - processing already");
  //                   $receipt_post->post_content = wp_kses_post($receipt_post->post_content . " - Order {$order_id} is already processing");
  //                   wp_update_post( $receipt_post );
  //                   http_response_code(200);
  //                 }
  //               }
  //               http_response_code(500);
  //             }
  //           } else {
  //             echo wp_kses_post(" <br> Order {$order_id} not found");
  //             if ( $receipt_post_id ) {
  //               $receipt_post = get_post( $receipt_post_id );
  //               if ( $receipt_post ) {
  //                 $receipt_post->post_title = wp_kses_post($receipt_post->post_title . " - order not found");
  //                 $receipt_post->post_content = wp_kses_post($receipt_post->post_content . " - Order {$order_id} was not found");
  //                 wp_update_post( $receipt_post );
  //                 http_response_code(200);
  //               }
  //             }
  //             http_response_code(404);
  //           }
  //         } else {
  //           echo wp_kses_post(" <br> Missing important data");
  //           if ( $receipt_post_id ) {
  //             $receipt_post = get_post( $receipt_post_id );
  //             if ( $receipt_post ) {
  //               $receipt_post->post_title = wp_kses_post($receipt_post->post_title . " - missing data error");
  //               $receipt_post->post_content = wp_kses_post($receipt_post->post_content . " - Missing data: Venmo ID: $venmo_username or  amount: $amount or order_id: $order_id. Received Email Subject: {$email_subject}");
  //               wp_update_post( $receipt_post );
  //               http_response_code(200);
  //             }
  //           }
  //           http_response_code(400);
  //         }
  //       } else {
  //         echo wp_kses_post(" <br> UPGRADE TO UNLOCK AUTOMATED ORDER PROCESSING");
  //         if ( $receipt_post_id ) {
  //           $receipt_post = get_post( $receipt_post_id );
  //           if ( $receipt_post ) {
  //             $receipt_post->post_title = wp_kses_post($receipt_post->post_title . " - UPGRADE TO AUTOMATE FURTHER");
  //             $receipt_post->post_content = wp_kses_post($receipt_post->post_content . " - Upgrade to automated further. Received Email Subject: {$email_subject}");
  //             wp_update_post( $receipt_post );
  //             http_response_code(200);
  //           }
  //         }
  //         http_response_code(400);
  //       }
  //     } else {
  //       if ( post_type_exists( 'venmo-receipts' ) ) {
  //         $forwarding_receipt = array(
  //           'post_title' => wp_kses_post("Forwarded: {$email_subject}"),
  //           'post_content' => wp_kses_post($email_body),
  //           'post_type' => 'venmo-receipts',
  //           'post_status' => 'draft',
  //         );
  //         $forwarding_post_id = wp_insert_post( $forwarding_receipt );
  //         if ($forwarding_post_id) {
  //           http_response_code(201);
  //         } else {
  //           http_response_code(500);
  //         }
  //       }
  //       http_response_code(422);
  //     }
  //   } else {
  //     if ( post_type_exists( 'venmo-receipts' ) ) {
  //       $forwarding_receipt = array(
  //         'post_title' => wp_kses_post("Forwarded: {$email_subject}"),
  //         'post_content' => wp_kses_post($email_body),
  //         'post_type' => 'venmo-receipts',
  //         'post_status' => 'draft',
  //       );
  //       $forwarding_post_id = wp_insert_post( $forwarding_receipt );
  //       if ($forwarding_post_id) {
  //         http_response_code(201);
  //       } else {
  //         http_response_code(500);
  //       }
  //     }
  //     http_response_code(403);
  //   }
  // }

  // function wc_venmo_check_email( $sender_email, $receiver_email ) {
  //   $email = wp_kses_post($this->ReceiverVENMOEmail);
  //   $venmo_email = "venmo@venmo.com";
  //   if ( $email == $receiver_email && $venmo_email == $sender_email ) {
  //     return true;
  //   }
  //   return false;
  // }

}

$WC_Venmo_Update_Order = new WC_Venmo_Update_Order();
$WC_Venmo_Update_Order->register();

endif;